(function() {
  var CSON, Clean, Command, _, async, config, fs, path, yargs;

  path = require('path');

  async = require('async');

  CSON = require('season');

  yargs = require('yargs');

  _ = require('underscore-plus');

  Command = require('./command');

  config = require('./apm');

  fs = require('./fs');

  module.exports = Clean = (function() {
    class Clean extends Command {
      constructor() {
        super();
        this.atomNpmPath = require.resolve('npm/bin/npm-cli');
      }

      getDependencies(modulePath, allDependencies) {
        var dependencies, error, i, installedModule, len, modulesPath, packageDependencies, ref, ref1, results;
        try {
          ({dependencies, packageDependencies} = (ref = CSON.readFileSync(CSON.resolve(path.join(modulePath, 'package')))) != null ? ref : {});
        } catch (error1) {
          error = error1;
          return;
        }
        _.extend(allDependencies, dependencies);
        modulesPath = path.join(modulePath, 'node_modules');
        ref1 = fs.list(modulesPath);
        results = [];
        for (i = 0, len = ref1.length; i < len; i++) {
          installedModule = ref1[i];
          if (installedModule !== '.bin') {
            results.push(this.getDependencies(path.join(modulesPath, installedModule), allDependencies));
          }
        }
        return results;
      }

      getModulesToRemove() {
        var dependencies, devDependencies, filteredInstalledModules, i, installedModule, installedModules, j, len, len1, modulePathFilter, modulesPath, modulesToRemove, packageDependencies, packagePath, ref;
        packagePath = CSON.resolve('package');
        if (!packagePath) {
          return [];
        }
        ({devDependencies, dependencies, packageDependencies} = (ref = CSON.readFileSync(packagePath)) != null ? ref : {});
        if (devDependencies == null) {
          devDependencies = {};
        }
        if (dependencies == null) {
          dependencies = {};
        }
        if (packageDependencies == null) {
          packageDependencies = {};
        }
        modulesToRemove = [];
        modulesPath = path.resolve('node_modules');
        modulePathFilter = function(modulePath) {
          return modulePath !== '.bin' && modulePath !== 'atom-package-manager';
        };
        installedModules = fs.list(modulesPath).filter(modulePathFilter);
        // Check if the module is a scoped module (starting with an '@')
        // If so, recursively lookup inside this directory
        // and concatenate to the root folder

        // e.g. if you have a dependency @types/atom, modulePath === @types
        // fs.list(@types) === ['atom'], thus this will return ['@types/atom']

        // At the end, flat map, since these scoped packages can return more than 1
        // and normal modules return only 1
        filteredInstalledModules = [].concat.apply([], installedModules.map(function(modulePath) {
          if (!(modulePath.substring(0, 1) === '@')) {
            return [modulePath];
          } else {
            return fs.list(path.join(modulesPath, modulePath)).filter(modulePathFilter).map(function(subPath) {
              return path.join(modulePath, subPath);
            });
          }
        }));
// Find all dependencies of all installed modules recursively
        for (i = 0, len = filteredInstalledModules.length; i < len; i++) {
          installedModule = filteredInstalledModules[i];
          this.getDependencies(path.join(modulesPath, installedModule), dependencies);
        }
// Only remove dependencies that aren't referenced by any installed modules
        for (j = 0, len1 = filteredInstalledModules.length; j < len1; j++) {
          installedModule = filteredInstalledModules[j];
          if (dependencies.hasOwnProperty(installedModule)) {
            continue;
          }
          if (devDependencies.hasOwnProperty(installedModule)) {
            continue;
          }
          if (packageDependencies.hasOwnProperty(installedModule)) {
            continue;
          }
          modulesToRemove.push(installedModule);
        }
        return modulesToRemove;
      }

      parseOptions(argv) {
        var options;
        options = yargs(argv).wrap(100);
        options.usage("Usage: apm clean\n\nDeletes all packages in the node_modules folder that are not referenced\nas a dependency in the package.json file.");
        return options.alias('h', 'help').describe('help', 'Print this usage message');
      }

      removeModule(module, callback) {
        process.stdout.write(`Removing ${module} `);
        return this.fork(this.atomNpmPath, ['uninstall', module], (...args) => {
          return this.logCommandResults(callback, ...args);
        });
      }

      run(options) {
        var doneCallback, uninstallCommands;
        uninstallCommands = [];
        this.getModulesToRemove().forEach((module) => {
          return uninstallCommands.push((callback) => {
            return this.removeModule(module, callback);
          });
        });
        if (uninstallCommands.length > 0) {
          doneCallback = (error) => {
            if (error != null) {
              return options.callback(error);
            } else {
              return this.run(options);
            }
          };
        } else {
          doneCallback = options.callback;
        }
        return async.waterfall(uninstallCommands, doneCallback);
      }

    };

    Clean.commandNames = ['clean'];

    return Clean;

  }).call(this);

}).call(this);
